package org.sitebricks.decorator.freemarker;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.util.FileUtils;
import org.sitebricks.extractor.ExtractResult;
import org.sitebricks.extractor.XhtmlExtractor;
import org.sitebricks.extractor.jsoup.JSoupXhtmlExtractor;

@Named
@Singleton
public class BrickSource implements ContextSource {

  private XhtmlExtractor extractor;
  private File bricksDirectory; 
  
  public BrickSource(File bricksDirectory) {
    this.bricksDirectory = bricksDirectory;
    extractor = new JSoupXhtmlExtractor();
  }
  
  public Map<String,Object> source() {    
    Map<String, Object> brickVariables = new HashMap<String,Object>();
    if (bricksDirectory != null && bricksDirectory.exists()) {
      try {        
        List<String> files = FileUtils.getFileNames(bricksDirectory, "*-brick.html", null, false);
        for (String fileName : files) {
          File brickFile = new File(bricksDirectory, fileName);
          String brickId = brickFile.getName().substring(0, brickFile.getName().indexOf("-brick"));
          ExtractResult result = extractor.extract(brickFile);
          //
          // The brick might contain text to inject in the head element or body element
          //
          if(result.getBody() != null && result.getBody().trim().length() != 0) {
            brickVariables.put(brickId, result.getBody());            
          } else if (result.getHead() != null && result.getHead().trim().length() != 0) {
            brickVariables.put(brickId, result.getHead());
          }          
        }                
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    }    
    return brickVariables;
  }
}
