package org.sitebricks.decorator.freemarker;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Map;

import javax.inject.Named;
import javax.inject.Singleton;

import org.codehaus.plexus.util.IOUtil;
import org.sitebricks.decorator.Decorator;

import com.github.mustachejava.DefaultMustacheFactory;
import com.github.mustachejava.Mustache;
import com.github.mustachejava.MustacheFactory;
import com.google.common.io.CharStreams;

@Named
@Singleton
public class FreemarkerDecorator implements Decorator {

  public void decorate(Reader decoratorTemplate, Map<String, Object> context, Writer writer) {  
    Reader reader = null;
    
    try {
      StringWriter decoratorTemplateText = new StringWriter();
      CharStreams.copy(decoratorTemplate, decoratorTemplateText);
      //
      // Interpolate the bricks into the page template
      //
      StringWriter interpolated = new StringWriter();
      MustacheFactory mf = new DefaultMustacheFactory();
      Mustache mustache = mf.compile(new StringReader(decoratorTemplateText.toString()), "bricks");
      mustache.execute(interpolated, context);
      reader = new StringReader(interpolated.toString());
    } catch (IOException e) {
      throw new RuntimeException(e);
    }

    //
    // Interpolate and process the page template that has the expanded bricks. This allows all known variables
    // to be used in the bricks which is intuitive to users.
    //    
    try {      
      MustacheFactory mf = new DefaultMustacheFactory();
      Mustache mustache = mf.compile(reader, "page");
      mustache.execute(writer, context);
    } finally {
      IOUtil.close(decoratorTemplate);
    }
  }
}
