package org.sitebricks.decorator.freemarker;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Map;

import javax.inject.Named;
import javax.inject.Singleton;

import org.sitebricks.decorator.Decorator;

import com.github.mustachejava.DefaultMustacheFactory;
import com.github.mustachejava.Mustache;
import com.github.mustachejava.MustacheFactory;
import com.google.sitebricks.Template;
import com.google.sitebricks.compiler.TemplateCompiler;
import com.google.sitebricks.compiler.template.AbstractMagicTemplateCompiler;

/**
 * Creates renderables, given a Freemarker decorator template and simple xhtml content that is parsed by JSoup. We take the title, head, body, and links as parsed from JSoup and inject them into the
 * template context as ${title}, ${head}, ${body}, and ${links}.
 */
@Singleton
@Named("dml")
public class FreemarkerDecoratorTemplateCompiler extends AbstractMagicTemplateCompiler implements TemplateCompiler {

  private final Decorator decorator;

  public FreemarkerDecoratorTemplateCompiler() {
    this.decorator = new FreemarkerDecorator();
  }

  @Override
  public com.google.sitebricks.Template transform(com.google.sitebricks.Template template) {
    //
    // Whatever, change markdown to XHTML
    //
    return template;
  }

  @Override
  public String process(Class<?> page, Object bound, Template sitebricksTemplate) {    
    Writer writer = new StringWriter();
    //File decoratorSource = new File(new File(sitebricksTemplate.getTemplateSource().getLocation()).getParentFile(), "template.html");
    File userDirectory = new File(System.getProperty("user.dir"));
    File decoratorSource = new File(userDirectory, "src/main/webapp/template.html");
    File bricksDirectory = new File(decoratorSource.getParentFile(), "bricks");

    ContextSource brickSource = new BrickSource(bricksDirectory);
    Map<String, Object> context = brickSource.source();
    Object[] scopes = new Object[] { bound, context };
    
    if (decoratorSource.exists()) {
      try {
        MustacheFactory mf = new DefaultMustacheFactory();
        Mustache mustache = mf.compile(new StringReader(sitebricksTemplate.getText()), "dummy");
        Writer pageWriter = new StringWriter();
        mustache.execute(pageWriter, scopes);
        // Xhtml of the page
        ContextSource xhtmlPageSource = new XhtmlPageSource(pageWriter.toString());
        context.putAll(xhtmlPageSource.source());
        decorator.decorate(new FileReader(decoratorSource), context, writer);
      } catch (FileNotFoundException e) {
        //
        // Won't happen
        //
      }
    } else {
      //
      // If the decorator template is missing we still want to attempt to inject the bricks. The user may just be getting started
      // and not have a decorator template yet, or the user may not want to use a decorator template for this page.
      // 
      decorator.decorate(new StringReader(sitebricksTemplate.getText()), context, writer);
    }

    return writer.toString();
  }  
}
